/*****************************************************
Workspace: CVAVR
Chip type: ATmega8
frequency: 8 MHz
Project  : PPGDT soldering station V2
Date     : Jan, 2017
*****************************************************/
#include <mega8.h>
#include <stdlib.h>
#include <math.h>
#include "defines.c"
#include "periph_init.c"
#include "keywork.c"
#include "display_work.c"
#include "ee_work.c"
#include "pwm_work.c"
#include "adc_work.c"
#include "pow.c"

u16 tim0, //msc 
    tim1, //1s timer
    tim2, //timeout_m
    tim3; //encoder dt 
u8  tim_s1;//200ms timer    
u16 tim_tcp;//tcp err timer 
    
bit iwdg_t2;

interrupt [TIM1_OVF] void t1_ovf_isr()
{
 OCR1A  = tmpA; 
 OCR1B  = tmpB;
 TIMSK &= ~(1<<TOIE1); 
 if(OCR1B!=cpu_speed/(pwm_freq*2)-OCR1A)PORTC.5=0;
}
interrupt [TIM2_COMP] void t2_ISR()
{
 static bit isr_trg;
 
 if(isr_trg)
 {
  isr_trg=0;
  scan_keys();   
 }
 else
 {
  isr_trg=1;
 }

 if(tim0)tim0--;
 if(tim1)tim1--; 
 if(tim2)tim2--; 
 if(tim3)tim3--; 
      
 if(tim_s1)tim_s1--;
 if(tim_tcp)tim_tcp--; 
 if(kb_timer)kb_timer--;
 
 iwdg_t2=1;
}

void devcon();

void main(void)
{
 gpio_init();
 t1_init(); 
 t2_init();
 adc_init();
 ee_init();
 detect_7seg_polarity(); 

 #pragma optsize-
 WDTCR=0x18;  //16ms
 WDTCR=0x08;
 #ifdef _OPTIMIZE_SIZE_
 #pragma optsize+
 #endif

 #asm("sei")         
      
 infinitely()
 { 
  devcon();
  display_data();    
  
  if(iwdg_t2){iwdg_t2=0; #asm("wdr")}
 }
}

void devcon()
{
 static enum {start,norm,sdn,tc_err,sc_prot,timeout_sdn,setup}dev_st=start;
 static bit itr=0;
 static u16 iron_t_def; 
 static u16 iron_t_tmp; 
 static struct my_structure pm;
 u8 i; 
 //u8 tmp_8;
 u16 tmp;
 u32 tmp_32;
 
 static bit skb_trg; 
 if(btn[0]==rls && btn[1]==rls)skb_trg=0;

switch(dev_st)
{
 case start:
 {
  static u8 sts;
  if(!itr)
  {
   itr=1;   
   dbuf[2]='t';dbuf[1]='c';dbuf[0]=0;dpp=1;
   tim0=1000;
   boost_trg=1; 
   set_pwm(0); 
   sts=0;
   pm = ee;
  }

  if(!tim0)
  {
   for(i=0;i<noc;i++)
   if(btn[i]==prs||btn[i]==hold)
   {
    dev_st=setup;
    skb_trg=1;
    goto endofstart;
   }  
   
   if(!sts)
   {
    if(read_ambt(&t_ambient, pm.Ct_diode, pm.Cv_diode))
    {  
#ifdef no_ambient_check 
     t_ambient = AMBIENT; 
#endif        
     convert_data(t_ambient/10); 
     dbuf[0]='C',dpp=1; 
     tim0=1000;    
     sts=1; 
    }
   } 
   else if(sts==1)
   {
    iron_t_def = ee_trd ()*10;
    convert_data(iron_t_def);
    dpp=0;
    tim0=1000; 
    sts=2;
   }
   else dev_st=norm;
  }
endofstart:  
  if(dev_st!=start){itr=0;}      
 }
 break;

 case norm:                 
 {
  static u8  timeout_m;
  static u16 adc_data[CO];
  static u16 dtmp;
  static u8 cmp2ee; 
  static bit ee_wt;
  static u8 m1_tim;
  static bit m1_switch;
                        
  if(!itr)
  {
   itr=1;
   
   timeout_m = 0xff;
   cmp2ee = ee_trd();
   iron_t_def = cmp2ee*10;  
   tim3 = 2000;
   tim_tcp = 10000; 
  }  
  
  kb_input(&iron_t_def,100,450,50,5,0xff); 

#asm("cli")  
  if(iron_t_def!=iron_t_tmp) //   t
  {
   iron_t_tmp = iron_t_def;
   tim3 = 2000;
   ee_wt=1; 
   timeout_m = 0xff; 
  }  
#asm("sei")  

  if(m1_switch)
  {
   if(read_ambt(&t_ambient, pm.Ct_diode, pm.Cv_diode)) 
   { 
    m1_switch = 0; 
   }
  }
  else
  {    
    if(read_adc(&tmp,1,0))  //   
    {
     tmp = tmp*pm.cpadc;          //  *100 
     
     if(!tim_tcp) dev_st=tc_err; //  
     else
     {
      if(tmp>pm.offset && tmp<MAXTC-1000) tim_tcp=10000;  //      
      
      for(i=CO-1;i;)adc_data[i]=adc_data[--i]; //  
      if(tmp+t_ambient>pm.offset)
      adc_data[0] = tmp+t_ambient-pm.offset;
      else adc_data[0] = t_ambient; 
      
      tmp_32=0;                
      for(i=0;i<CO;)tmp_32+=adc_data[i++];        // 
      tmp_32/=(sizeof(adc_data)/sizeof(i));
     
      set_pwm(t_stab_proc(iron_t_tmp, adc_data)); 
     }
   
     if(!tim3)     //       
     { 
      if(dtmp!=tmp || !dpp)
      {
       dtmp=tmp;
       dpp=5; 
       convert_data(tmp_32/100);
      }
      
      if(ee_wt)                            //   eeprom
      { 
       ee_wt = 0;
       if(iron_t_tmp/10!=cmp2ee)ee_t_wt(iron_t_tmp/10); 
      }
     }
     else 
     {
      dpp=0;
      convert_data(iron_t_tmp);  //     
     }
    }
    
    if(!tim_s1)          //255ms 
    { 
     tim_s1 = 0xff;
     read_adc(&tmp,1,1); //   
    }
  }
  
  if(!tim2)            //1m 
  {
   tim2 = 60000; 
   
   if(!m1_tim)
   {
    m1_tim = 5;
    m1_switch = 1; 
   }
   else m1_tim--;    
   
   if(timeout_m)timeout_m--;
   else dev_st = timeout_sdn;  
  }
  if(dev_st!=norm){itr=0;}  
 }
 break;

 case sdn:
 {
  if(!itr)
  {
   itr=1;
   set_pwm(0); 
   iron_t_tmp=iron_t_def;   
   dbuf[2]=dbuf[1]=dbuf[0]='_';dpp=5;   
  } 
  
  kb_input(&iron_t_tmp,100,450,50,5,0xff); 
  if((iron_t_tmp>iron_t_def+49)||(iron_t_tmp<iron_t_def-49)) dev_st=start;   

  if(dev_st!=sdn){itr=0;}    
 }
 break;  
 
 case tc_err:
 {
  static u16 pp;
  if(!itr)
  {
   itr=1;  
   dbuf[2]='t';dbuf[1]='c';dbuf[0]='E';dpp=1;
   pp = 10;
   set_pwm(0);  
  }   
  
  tmp=pp;
  kb_input(&pp,10,99,10,1,0xff); 
  
  if(tmp!=pp)
  {
   convert_data(pp); 
   dpp=3; dbuf[2]='P';
   set_pwm(pp*0xff/99);
  }   

  if(dev_st!=tc_err){itr=0;}   
 } 
 break;

 case sc_prot:
 {

  if(!itr)
  {
   itr=1;
   set_pwm(0);  
   dbuf[2]='S';dbuf[1]='c';dbuf[0]='P';dpp=1;    
  }  

  if(dev_st!=sc_prot){itr=0;}   
 } 
 break;
 
 case timeout_sdn:
 {
  if(!itr)
  {
   itr=1;
   set_pwm(0);
   iron_t_tmp=iron_t_def; 
   dbuf[2]='t';dbuf[1]='o';dbuf[0]='t';dpp=5;    
  }  
  
  kb_input(&iron_t_tmp,100,450,50,5,0xff); 
  
  if((iron_t_tmp>iron_t_def+49)||(iron_t_tmp<iron_t_def-49)) dev_st=start;
  if(dev_st!=timeout_sdn){itr=0;}   
 } 
 break;
 
 case setup:
 {
  static enum {set_amp, set_ambient, set_offset}setting;
  static bit stt_trg; 
  static bit read_ambient_trg;
  if(!itr)
  {
   itr=1;
   dbuf[2]='S';dbuf[1]='E';dbuf[0]='t';dpp=5; 
   tim0=1000;   
   setting=set_amp; 
   stt_trg=0;
   tim3 = 10000; 
   read_ambient_trg=1;
  } 
  
  if(read_adc(&tmp,0,0)) 
  {
   pm.Cv_diode = (u32)tmp*ADCMAX/0x3ff;
   read_ambient_trg=1; 
  }  
   
  if(btn[0]==prs && btn[1]==prs && !skb_trg)
  { 
   skb_trg=1;   
   stt_trg=0;
   if(setting==set_amp)setting=set_ambient;
   else if(setting==set_ambient) setting=set_offset; 
   else setting=set_amp;
  }
  
  if(!tim0)
  { 
   if(!tim3){dev_st=start; goto endofsetup;}
   if(setting==set_amp)
   { 
    if(!stt_trg)
    {
     stt_trg=1;
     dbuf[2]='A';dbuf[1]='P';dbuf[0]='f';dpp=5; 
    }
    tmp=pm.k_amplifier; 
    if(!skb_trg)kb_input(&pm.k_amplifier,10000,20000,100,100,0xff/2); 
    if(tmp!=pm.k_amplifier) 
    {
     tim3 = 5000;
     convert_data(pm.k_amplifier/100);
    }    
   }
   else if(setting==set_ambient)
   {
    if(!stt_trg)
    {
     stt_trg=1;   
     read_ambient_trg=0;   
     read_adc(&tmp,0,1);
     dbuf[2]='A';dbuf[1]='b';dbuf[0]='t';dpp=5;        
    } 
    
    tmp=pm.Ct_diode;
    if(!skb_trg)kb_input(&pm.Ct_diode,0,6000,100,100,0xff/2); 
    if(tmp!=pm.Ct_diode)
    {
     tim3 = 5000;
     convert_data(pm.Ct_diode/10),dbuf[0]='C',dpp=1;
    }
   } 
   else
   {
    if(!stt_trg)
    {
     stt_trg=1;
     dbuf[2]='O';dbuf[1]='f';dbuf[0]='S';dpp=5;        
    }
    
    tmp=pm.offset;
    if(!skb_trg)kb_input(&pm.offset,0,9000,100,100,0xff/2); 
    if(tmp!=pm.offset)
    {
     tim3 = 5000;
     convert_data(pm.offset/10),dbuf[0]='C',dpp=1;
    }     
   }   
  } 

endofsetup: 
  if(dev_st!=setup && read_ambient_trg)
  {
   itr=0;
   ee.k_amplifier   = pm.k_amplifier;
   ee.Ct_diode      = pm.Ct_diode;
   ee.Cv_diode      = pm.Cv_diode; 
   ee.offset        = pm.offset;
   ee.maxtc         = ADCMAX*1000*100/K_PAIR*100/pm.k_amplifier;
   ee.cpadc         = ee.maxtc/0x3ff;
  }
 } 
 break;   
}
}